<?php
/** @noinspection PhpMissingFieldTypeInspection */
declare(strict_types=1);

namespace Inti\Provider\Service;

use Doctrine\Common\Collections\ArrayCollection;
use Doctrine\Common\Collections\Collection;
use Inti\Provider\ProviderInterface;
use Inti\Provider\Type\TypeInterface;
use Laminas\ServiceManager\AbstractPluginManager;

use function array_unique;
use function boolval;
use function get_class;
use function is_integer;

/**
 * Class ProviderManager
 *
 * @package Inti\Provider\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/oktan-nautika/inti-provider
 *
 * @method ProviderInterface|object get(string $id)
 */
final class ProviderManager extends AbstractPluginManager implements ProviderManagerInterface
{
    protected $instanceOf = ProviderInterface::class;

    private bool       $autoregister_providers = false;
    private Collection $providers;
    private array      $registered;

    public function __construct($configInstanceOrParentLocator = null, array $config = [])
    {
        if (isset($config['autoregister_providers'])) {
            $this->autoregister_providers = boolval($config['autoregister_providers']);
        }

        $this->registered = $config['registered'] ?? [];

        parent::__construct($configInstanceOrParentLocator, $config);
    }

    /**
     * @param string $type
     * @param TypeInterface $providedType
     * @return bool
     */
    public function canProvide(string $type, TypeInterface $providedType): bool
    {
        return $this->getProvider($type, $providedType) instanceof ProviderInterface;
    }

    /**
     * @param string $type
     * @param TypeInterface $providedType
     * @return ProviderInterface|null
     */
    public function getProvider(string $type, TypeInterface $providedType): ?ProviderInterface
    {
        return $this->getProviders($type, $providedType)[0] ?? null;
    }

    /**
     * @param string|null $type
     * @param TypeInterface|null $providedType
     * @return ProviderInterface[]
     */
    public function getProviders(?string $type = null, ?TypeInterface $providedType = null): array
    {
        return $this->getAllRegisteredProviders()->filter(function (ProviderInterface $provider) use ($type, $providedType) {
            if (empty($type) && empty($providedType)) {
                return true;
            }

            if (!empty($type) && !$provider instanceof $type) {
                return false;
            }

            if (!empty($providedType) && $provider->getProvidedType() !== $providedType) {
                return false;
            }

            return true;
        })->getValues();
    }

    /**
     * @return Collection|ProviderInterface[]
     */
    private function getAllRegisteredProviders(): Collection
    {
        if (!isset($this->providers)) {
            $this->providers = new ArrayCollection();

            $lists = $this->autoregister_providers
                ? [$this->services, $this->factories]
                : [$this->registered];

            foreach ($lists as $list) {
                foreach ($list as $id => $value) {
                    if (is_integer($id)) {
                        $id = $value;
                    }

                    $provider = $this->get($id);

                    if (!$this->providers->contains($provider)) {
                        $this->providers->add($provider);
                    }
                }
            }
        }

        return $this->providers;
    }

    public function registerProvider($provider): void
    {
        if ($provider instanceof ProviderInterface) {
            $providerName = get_class($provider);
            $this->setService($providerName, $provider);
            $provider = $providerName;
        }

        $this->registered[] = $provider;
        $this->registered = array_unique($this->registered);

        unset($this->providers);
    }

    public function registerProviders(array $providers): void
    {
        foreach ($providers as $provider) {
            $this->registerProvider($provider);
        }
    }
}
